import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {action, computed} from "mobx";
import {ScannerRuleModel} from "./ScannerRuleStore";
import {ScannerType} from "./ScannerOptionStore";

@model({
    transportLayer: '/api/scanners',
    permissionTarget: "Scanner",
    routingInfo: {
        create: '/scanners/create',
        bulkCreate: '/scanners/bulk_create',
        read: '/scanners/:id/edit',
        list: '/scanners',
    }
})

export class ScannerModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Scanner name is a required field`)
    })
    name: string = '';

    @attribute({
        label: t`Type`,
        validation: yup.string().required(t`Scanner Type is a required field`)
    })
    type: ScannerType | null = null;

    @attribute({
        label: t`IP Address`,
        validation: yup.string().matches(/^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$/, t`Must be an ip address`)
    })
    ip_address: string = '';

    @attribute({
        label: t`Last Read Value`,
        validation: yup.string().nullable()
    })
    last_read_value: string = '';

    @attribute({
        label: t`Last Read Time`,
        validation: yup.string().nullable()
    })
    last_read_time: string = '';

    @attribute({})
    parameters: Record<string, unknown> = {};

    @attribute({})
    rules_to_add: ScannerRuleModel[] = []

    @action
    async ping() {
        await this.store.transportLayer.fetch(`/${this.id}/ping_v2`);
    }

    @action
    async scan() {
        const response = await this.store.transportLayer.fetch(`/${this.id}/scan`);

        const data = await response.json();

        this.last_read_value = data.last_read_value;
        this.last_read_time = data.last_read_time;
    }

    @computed
    get last_scan_successful(): boolean {
        return this.last_read_value !== 'fail';
    }
}

export const ScannerStore = ScannerModel.store as DataStore<ScannerModel>;
