module ScannerHandler
  include ScannerConnection

  class InvalidLogLevel < StandardError; end
  class FailedToRead < StandardError; end

  class BaseScannerHandler
    def initialize(name, ip, port)
      @name = name
      @connection_client = ScannerConnection::ScannerConnectionWrapper.new(ip, port)

      @connection_mutex = Mutex.new
      @use_count = 0
    end

    def scan
      log("scanning")

      ensure_mutex do
        attempt ||= 0

        result = emit_scan

        log("scan results: #{result}")

        {success: true, value: result, time: Time.zone.now}
      rescue SystemCallError => err
        attempt += 1
        if attempt <= 3
          log("connection failed - retrying (attempt #{attempt}/3). Error: #{err}")
          close_connection
          retry
        else
          {success: false, error: "failed to open a connection to scanner after 3 attempts: #{err}", time: Time.zone.now}
        end
      rescue FailedToRead
        log("failed to read", :warn)
        {success: false, scan_failed: true, time: Time.zone.now}
      rescue StandardError => err
        log("scan failed: #{err}", :warn)

        {success: false, error: err, time: Time.zone.now}
      end
    end

    def ping
      log("pinging")

      ensure_mutex do
        @connection_client.ping
      end
    end

    def close_connection
      @connection_client.close
    end

    private

    def emit_scan
      raise NotImplementedError
    end

    concerning :Connection do
      def ensure_mutex
        log("acquiring mutex.")
        checkout_connection
        resp = nil

        begin
          @connection_mutex.synchronize do
            resp = yield
          end
        rescue => err
          log("Error during connection block #{err}", :error)
        ensure
          checkin_connection
        end

        resp
      end

      private

      def checkin_connection
        log("open connections: #{@use_count} => #{@use_count-1}")

        @use_count -= 1
      end

      def checkout_connection
        log("open connections: #{@use_count} => #{@use_count+1}")

        @use_count += 1

        if @use_count > 5
          log("open connection count exceeded! Closing all open connections.", :warn)
          close_client
        end
      end
    end

    concerning :Commands do
      def scan_command(start_pos=0)
        command = []
        telegram_length_addr = nil # telegram length is special, since it can only be generated at the very end
        telegram_length_encoding_length = 0 # the telegram length field takes space itself
        total_length = 0
        scan_sequence.each do |command_def|
          total_length = total_length + command_def[:bits]
          if command_def[:literal]
            command.push(encode_transmission(command_def[:literal], command_def[:bits]))
          elsif command_def[:code] == :start_position
            command.push(encode_transmission(start_pos, command_def[:bits]))
          elsif command_def[:code] == :telegram_length
            telegram_length_addr = command.length
            telegram_length_encoding_length = command_def[:bits]
            command.push(:telegram_length)
          else
            command.push(encode_transmission(self.send(command_def[:code]), command_def[:bits]))
          end
        end
        command[telegram_length_addr] = encode_transmission(total_length / 8, telegram_length_encoding_length) if telegram_length_addr
        bin_str_to_packed_hex_str(command.join) # sometimes multiple commands wind up in a single byte, and leading zeros are important, so be careful if modifying the encoding strategy
      end

      private

      def scan_sequence
        raise NotImplementedError
      end

      def encode_transmission(value, bits)
        value = value.unpack1("H*").to_i(16) if value.is_a?(String)
        sprintf("%0#{bits}b", value || 0).slice(0, bits)
      end

      def bin_str_to_packed_hex_str(str)
        chunk(str, 8).map { |b| b.to_i(2) }.pack('C*')
      end

      def chunk(string, size)
        (0..(string.length - 1) / size).map { |i| string[i * size, size] }
      end
    end

    def log(message, level=:info)
      raise InvalidLogLevel unless [:info, :warn, :error].include?(level)
      Rails.logger.send(level, "#{@name} : #{message}")
    end
  end
end



