class Scanner < ApplicationRecord
  has_many :scanner_rules

  # These require dependency are needed to be able to do file class introspection from the top level
  # to see from the parent class what are it's subclasses.
  require_dependency 'scanner_types/datalogic_matrix120'
  # require_dependency 'scanner_types/pepperl_fuchs_iqh1_fp_v1'
  require_dependency 'scanner_types/sick_clv62x'
  require_dependency 'scanner_types/wenglor_b50'

  class << self
    def options
      @options ||= class_name_to_make_and_model_hash.map {|k, v| {make: v[0], model: v[1]}}
      @options
    end

    def make_and_models_hash
      return @make_and_model_hash unless @make_and_model_hash.nil?
      @make_and_model_hash = {}
      class_name_to_make_and_model_hash.each do |k, v|
        @make_and_model_hash[v[0]] ||= {}
        @make_and_model_hash[v[0]][v[1]] = k
      end
      @make_and_model_hash
    end

    def make_model_to_type(make, model)
      make_and_models_hash[make][model]
    end

    def class_name_to_make_and_model_hash
      return @class_name_to_make_model_hash unless @class_name_to_make_model_hash.nil?
      @class_name_to_make_model_hash = {}
      subclasses.map do |subclass|
        @class_name_to_make_model_hash[subclass.name] = [subclass.make, subclass.model]
      end
      @class_name_to_make_model_hash
    end

    def get_make_model_from_type(type)
      class_name_to_make_and_model_hash[type]
    end

    def make
      raise NotImplementedError
    end

    def model
      raise NotImplementedError
    end
  end

  def add_rules(rule_params)
    rule_params.each do |rule_param|
      # add_to_target adds to the collection without saving.
      association(:scanner_rules).add_to_target(ScannerRule.new(rule_param))
    end
  end

  def scanner_type
    make, model = Scanner.get_make_model_from_type(type)
    "#{make}/#{model}"
  end

  def scan_handler
    ScannerManager.handler_for_model(self)
  end

  def ping
    handler = scan_handler
    handler.ping
  end

  def scan
    handler = scan_handler

    response = handler.scan

    if response[:success]
      update!(last_read_time: Time.zone.now, last_read_value: apply_scan_rules( response[:value]))
    else
      if response[:scan_failed]
        update!(last_read_time: Time.zone.now, last_read_value: 'fail')
      end
    end

    response
  end

  def apply_scan_rules(result)
    rules = scanner_rules.to_a

    return result if rules.size == 0

    rules.sort_by {|rule| rule.priority}
    rules.each do |rule|
      if rule.operation == 'delete'
        result.slice!(rule.start_position, rule.end_position)
      elsif rule.operation == 'append'
        result << rule.value
      elsif rule.operation == 'prepend'
        result.prepend(rule.value)
      end
    end
    result
  end
end